@@ -15,3 +15,11 @@ BUYBACKPROGRAM_TASKS_TIME_LIMIT = clean_setting("BUYBACKPROGRAM_TASKS_TIME_LIMIT
 BUYBACKPROGRAM_TRACKING_PREFILL = clean_setting(
     "BUYBACKPROGRAM_TRACKING_PREFILL", "aa-bbp"
 )
+
+
+def allianceauth_discordbot_active():
+    """
+    check if allianceauth-dicordbot is installed and active
+    :return:
+    """
+    return "aadiscordbot" in settings.INSTALLED_APPS
